function [Q,W] = Gauss_SVD(A,tol)
%
%  function [Q,W] = Gauss_SVD(A,tol)
%  ================================= 
%
%  such that
%
%    QAW = [I 0]
%          [0 0]
%
%    tol is optional tolerance setting
%

%  (c) 2011 by Tomas Co
%  @ Michigan Technological University

   if nargin<2
       tol = 1e-10;
   end
   n = size(A,1);
   m = size(A,2);
   [U,S,V]=svd(A);
   k = min(n,m);
   SS = S(1:k,1:k);
   d = diag(SS);
   d(d<=tol)=1;
   d = 1./d;
   Sinv = diag(d);
   if n>m
       Q = U';
       W = V*Sinv;
   else
       Q = Sinv*U';
       W = V;
   end
   