function [R,P,Q] = QR_house(A)
%
%  function [R,P,Q] = QR_house(A)
%  ==============================
%     QR factorization with permutation
%     i.e. QR = AP
%     such that
%     R = upper triangular matrix
%           with diagonals in descending magnitude
%     Q'*Q = I
%   

% (c) 2010 by Tomas Co
% @ Michigan Technological University

[n, m] = size(A);
Q   = eye(n)                                            ;
R   = A                                                 ;
P   = 1:m                                               ;
mn  = min(n,m)                                          ;
for j = 1:mn - 1
    HH  = R(:,j:end)                                    ;
    H   = HH(j:end,:)                                   ;
%    if j< mn
        pp          = P(j:end)                          ;
        h           = sum(conj(H).*H)                   ;
        [im,ip]     = sort(h,'descend')                 ;
        H           = H(:,ip)                           ;
        P(j:end)    = pp(ip)                            ;
        R(:,j:end)  = HH(:,ip)                          ;
%    end
    x               = H(:,1)                            ;
    u               = x                                 ;
    alpha           = norm(x)                           ;
    if abs(alpha) < 1e-12
        break
    else
        u(1)            = u(1) - alpha                  ;
        w               = 2*u/(u'*u)                    ;
        z               = u'*H                          ;
        R(j:end,j:end)  = H - w*z                       ;
        z               = u'*Q(j:end,:)                 ;
        Q(j:end,:)      = Q(j:end,:) - w*z              ;
    end
end
Q               = Q'                                    ;
Q(:,m+1:end)    = []                                    ;
R(m+1:end,:)    = []                                    ;
I               = eye(m)                                ;
P               = I(:,P)                                ;
