function    [U,S,V] = SVD_alg(A,tol)
%
% function    [U,S,V] = SVD_alg(A,tol)
% ====================================
% 
%   U*S*V' = A
%       A [=] N x M, U [=] N x N, S [=] N x M, V [=] M x M
%       U,V unitary, S diag with singular values
%       optional: tol = tolerance (default: 1e-6)

%   (c) 2006 by Tomas Co
%   @ Michigan Technological University

%   Initialize                                                        ;
%   ==========                                                        ;
    maxiter = 1000                                                    ;    
    if nargin<2     ;    tol=1e-6       ;    end                     ; 
    N = size(A,1)   ;    M = size(A,2)   ;    
%                                                                     ;    
%   obtain V and singular values via QR method                        ;
%   ==========================================                        ;
%                                                                     ;
    AA = A'*A               ;    V = eye(size(AA))                    ;
    S = AA                  ;    iter =0                              ;
    y =  S- diag(diag(S))   ;    e = norm(y(:))                       ;
    while e>tol                                                       ;
        [Q,R]=qr(S)         ;    S = R*Q         ;   V = V*Q          ;
        y = S-diag(diag(S)) ;    e = norm( y(:)) ;   iter = iter + 1  ;
        if iter>maxiter                                               ;   
            disp('maximimum iteration exceeded') ;   return           ; 
        end                                                           ;
    end                                                               ;
    d = sqrt(abs(diag(S)))  ;    r = sum(d>tol)                       ;   
    [d,id] = sort(-d)       ;    d = -d                               ;    
    subd = d(1:r)           ;    V = V(id,:)                          ;
%                                                                     ;    
%   obtain U and S                                                    ;
%   ==============                                                    ;
%                                                                     ;
    V1 = V(:,[1:r])         ;    U = A*V1*[diag(subd.^(-1))]          ;
    if N>r;                                                           ;
        U =[U,rand(N,N-r)]  ;    [U,R] = QR_alg(U)                    ;
    end                                                               ;
    S = zeros(N,M)          ;    S([1:r],[1:r])=diag(subd)            ;