function [S,pn,eigs]=charpoly(A)
%
%   function [S,pn,eigs]=charpoly(A)
%   ================================
%
%   Based on Danilevskii's method,
%       S     = matrix such that inv(S)AS is companion form
%       pn    = cell array of polynomial coefficients
%               (i.e. to get the nth vector, type pn{n} )
%       eigs  = vector of eigenvalues

%  (c) 2009 by Tomas Co
%  @ Michigan Technological University

n = size(A,1)       ; In = eye(n)       ; S = In                ;     
tol = 1e-15         ; eigs = []                                 ;
if n==1                                                         ;
    pn{1} = [1,-A]      ; eigs = [A]                            ;
else                                                            ;
    for i=1:n-1                                                 ;
        P = In          ; PI = In       ; i1 = i+1              ;
        z = A(i,:)      ; q = z(i1)     ; pn = {}               ;
        [h,imx] = max( abs(z([i1:n])) )                         ;
        if h<tol                                                ;
            [T,y,lambda] = charpoly( A([i1:n],[i1:n]) )         ;
            zn = zeros(i,n-i)   ; T = [eye(i),zn;zn',T]         ;
            S = S*T             ; eigs = [lambda; eigs]         ;
            pn = [y;pn]         ; i = i-1                       ;
            break                                               ;
        elseif imx>1                                            ;
            A( :,[i1,imx+i] ) = A( :,[imx+i,i1] )               ;
            A( [i1,imx+i],: ) = A( [imx+i,i1],: )               ;
            S( :,[i1,imx+i] ) = S( :,[imx+i,i1] )               ;
            z = A(i,:)          ; q = z(i1)                     ;
        end                                                     ; 
        PI(i1,:) = z    ; P(i1,:) = -z/q    ; P(i1,i1) = 1/q    ; 
        A = PI*A*P      ; S = S*P                               ; 
    end                                                         ;
    cpoly = fliplr([-A(i+1,1:i+1),1])       ; pn = [cpoly;pn]   ;
    eigs = [ roots(cpoly); eigs ]                               ;
end                                                             ;