function [choles,rholes] = circ_arrays(rows,cols,dx,dy,r,acx,acy)
%
%   function [choles,rholes] = circ_arrays(rows,cols,dx,dy,r,acx,acy)
%   =================================================================
%
%       rows, cols = number of rows and cols
%       dx, dy = horizontal and verical distance
%       r = radius of circles
%       acx, acy = position of the center of the arrays
%
% 
%  Example:
%
%   [choles,rholes]   = circ_arrays(3,3,0.4,0.3,0.125,1,0.5)              ;
%   [x,y,tri]         = rect_circ_mesh([0;0],[2;1],choles,rholes,0.03)    ;
%   triplot(tri,x,y);  
%   axis equal;  shg

% (c)2011 by Tom Co
% @ Michigan Technological University
sw = 1 ;
choles = [];
for j=0:cols-1
    krem = (1-sw)/2;
    yadd = (1-sw)/4;
    nrows = rows - krem;
    cx = j*dx*ones(1,nrows);
    cy = ((0:nrows-1)+yadd)*dy;
    choles = [choles,[cx;cy]];
    sw = -sw;
end
rholes = r*ones(1,size(choles,2));
cx = (cols-1)*dx/2;
cy = (rows-1)*dy/2;
choles(1,:)=choles(1,:)-cx+acx;
choles(2,:)=choles(2,:)-cy+acy;
%


