function [x,y,tri] = circle_annulus(c,r,choles,rholes,dL)
%
% function [x,y,tri] = circle_annulus(c,r,choles,rholes,dL)
% =========================================================
%
%   Triangulation of annulus
%   Assumes holes are inside outer circle ( rholes < r1 )
%     c  : center points of outer circle
%     choles: center points of holes 
%           = [ c1x, c2x, ...  ]
%             [ c1y, c2y, ...  ]
%     r  : radius of outer circle
%     rholes : radius of holes = [r1, r2, ...]
%     dL       : approximate characteristic length of
%                finite element
%   

%   Build Outer Circle
%
    x = [];
    y = [];
    cx = c(1);
    cy = c(2);
    nsectors = 6;
    tth = 2*pi/nsectors ;
    for i = 1:nsectors
       th1 = (i-1)*tth;
       th2 = i*tth;
       [xa,ya] = circle_arc_mesh(r,th1,th2,dL);
       thedge  = atan2(ya,xa); 
       on_edge = (abs( th2 - thedge - (thedge<0)*2*pi)<=1e-6); 
       at_origin = (xa==0 & ya==0);
       xa = xa(~on_edge & ~at_origin);
       ya = ya(~on_edge & ~at_origin);
       x = [x,xa];
       y = [y,ya];
    end
    x=[0,x];
    y=[0,y];
    x = x + cx;
    y = y + cy;
%
%   Remove holes
%
    nholes = size(choles,2);
    for i=1:nholes
        r2 = rholes(i);
        c2x = choles(1,i);
        c2y = choles(2,i);
        d_from_hole_center = sqrt( (x-c2x).^2 + (y-c2y).^2 ); 
        inside_hole =   d_from_hole_center < (r2 + dL/6) ;
        x = x(~inside_hole);
        y = y(~inside_hole);
        nth = ceil( 2*pi*r2/(0.99*dL) );
        th = linspace(0,2*pi,nth);
        th(end)=[];
        xhole = r2*cos(th)+c2x;
        yhole = r2*sin(th)+c2y;
        x =[x,xhole];
        y =[y,yhole];
    end
    tri = delaunay(x,y);
    keep = ones(size(tri,1),1);
    for i=1:nholes
        r2 = rholes(i);
        c2x = choles(1,i);
        c2y = choles(2,i);
        d_from_hole_center = sqrt( (x-c2x).^2 + (y-c2y).^2 ); 
        at_hole_boundary =   d_from_hole_center < (r2 + 1e-8) ;
        keep = keep & sum(at_hole_boundary(tri),2)<3;
    end
    tri = tri(keep,:);