function    x = conj_grad(A,b,x0)
%
%	function    x = conj_grad(A,b,x0)
%	=================================
%
%		conjugate gradient method
%
%		for Ax=b
%
%		initial guess is x0

%	(c) 2006 by Tomas Co
%	@ Michigan Technological University
%
	
	tol     = 1e-12                 ;
	cnt     = 0                     ;
	maxcnt  = 50                    ;
	iA      = issparse(A)           ;
	A       = sparse(A)             ;
    b       = sparse(b)             ;
	n       = size(A,1)             ;
	if nargin==2
    	x   = b                     ;
    else
        x   = x0                    ;
        x   = sparse(x)             ;
    end
    r       = b - A*x               ;
    d       = r                     ;
    rr      = r'*r                  ;

    for i=1:n*2
        ad      = A*d               ;
        dad     = d'*ad             ;
        alpha   = rr/dad            ;
        x       = x + alpha*d       ;
        if cnt > maxcnt
            r   = b - A*x           ;
            cnt = 0                 ;
        else
            r   = r - alpha*ad      ;
            cnt = cnt + 1           ;
        end
        nrr     = r'*r              ;
        beta    = nrr/rr            ;
        d       = r + beta*d        ;
        rr      = nrr               ;
        if nrr<tol
            break
        end
    end
    if iA==0
        x   = full(x)               ;
    end