function [L,U,R] = crout_rowpiv(A,tol)
%
% function [L,U,R] = crout_rowpiv(A,tol)
% ========================================
%       Crout's method with partial pivoting, 
%       i.e.            LU = A(R,:)
%
%       Optional: tol= tolerance (default: tol=1e-12)

%   (c) 2006,2009 by Tom Co
%   @ Michigan Technological University

% Initialize
% ----------                                                           
    if nargin<2
        tol = 1e-12     ;
    end                                       
    spA = issparse(A)   ; 
    A   = sparse(A)     ;
    N   = size(A,1)     ; 
    U   = sparse(N,N)   ;
    L   = sparse(N,N)   ; 
    R   = 1:N           ;  
    p   = 1             ;
    iss = false         ;
%                                                                      
%                                                                    
% Algorithm proper  
% ----------------  
    L([1:N],1)  = A([1:N],1)                                ;
    [A,L,R]     = halfpivot(A,L,R,p)                        ;
    if abs(L(p,p))<tol
        [L,U,R] = singular()                                ;
    end
    U(1,:)      = [1,A(1,[2:N])/L(1,1)]                     ;
    for p = 2:N-1                
        Op      = 1:p-1                                     ; 
        PN      = p:N                                       ;
        L(PN,p) = A(PN,p)-L(PN,Op)*U(Op,p)                  ; 
        [A,L,R] = halfpivot(A,L,R,p);
        if abs(L(p,p))<tol 
            [L,U,R] = singular()                            ;
            return 
        end
        U(p,p)  = 1                                         ; 
        U(p,PN) = (A(p,PN)-L(p,Op)*U(Op,PN))/L(p,p)         ;
    end
    U(N,N)      = 1                                         ;
    L(N,N)      = A(N,N)-[L(N,[1:N-1])]*U([1:N-1],N)        ;
    if  abs(L(N,N))<tol; 
        p       = N-1                                       ; 
    else
        p       = N                                         ; 
    end
    [L,U]       = makefull(L,U,spA)                         ;

%
  function    [A,L,R] = halfpivot(A,L,R,p)                             
    %                                                     
    %   row-wise pivot to maximize |L(p,p)|               
    %   -----------------------------------               
    LN=L(p:end,p); [imax,i]=max( abs(LN) )                  ;
    if i>1    % (interchange rows)
       p1=[p, p+i-1]; p2=[p+i-1, p]                         ; 
       R(p1)=R(p2)  ; A(p1,:)=A(p2,:); L(p1,:) = L(p2,:)    ;
    end
%
  function [L,U] = makefull(L,U,spA)          
    %                                         
    %   if A was full, then make L and U full 
    %   ------------------------------------- 
    if ~spA 
        L   = full(L)                                       ;
        U   = full(U)                                       ; 
    end
%
      function [L,U,R]=singular()
          disp('Matrix is singular');
          L=[];
          U=[];
          R=[];