function    [x,f,del,NoCh,MXed] = dogleg( Fx,xc,fc,gc,... 
                    Lc,pn,del,MXStp,StpTol,nsolve,par)
%  function [x,f,del,NoCh,MXed] = dogleg( Fx,xc,fc,gc,...               ;
%                    Lc,pn,del,MXStp,StpTol,nsolve,par)                 ;
%  ===========================================================          ;
%                                                                       ;
%       double dogleg search for minimization stepsize                  ;
%       ( adapted from: Dennis and Schnabel, "Numerical                 ;
%           Methods for Unconstrained Optimization and                  ;
%           Nonlinear Equations", Prentice Hall, 1983 )                 ;

%   (c) 2007 by Tomas Co                                                ;
%   @ Michigan Technological Universtity                                ;

rcode = 0           ;   pary = false                                    ;
not_done = true     ;   firstPass = true    ;   NLen = norm(pn)         ;
alpha = 1e-4        ;   NoCh = false        ;   MXed = false            ;
if nargin <10       ;   nsolve = false      ;   end                     ;     
if nargin >10       ;   pary = true         ;   end                     ;
while not_done                                                          ;
    if NLen <= del  ;   NUsed=true          ;   p=pn   ;  del=NLen      ;
    else            ;   NUsed=false                                     ;
        if firstPass                                                    ;
            a=gc*gc'         ;  b=gc*Lc*Lc'*gc'        ;  pCP=-(a/b)*gc';       
            CPLen=norm(pCP)  ;  eta=0.2+0.8*(a^2/b)/abs(gc*pn)          ; 
            v=eta*pn-pCP     ;  firstPass=false                         ;
            if del == -1     ;  del=min(CPLen,MXStp)   ;   end          ;
        end                                                             ;
        if eta*NLen <= del   ;  p=(del/NLen )*pn                        ;
        elseif CPLen >= del  ;  p=(del/CPLen)*pCP                       ;
        else ;  a=v'*v       ;  b=v'*pCP    ; c=CPLen^2-del^2           ;
            ll = (-b+sqrt(b^2-a*c))/a       ; p=pCP + ll*v              ;
        end                                                             ;
    end                                                                 ;
    %           update trust region                                     ;
    %           -------------------                                     ;
    x = xc + p                                                          ;
    if pary   ; f=feval(Fx,x,par);   else   ;  f=feval(Fx,x) ;  end     ;
    if nsolve ; f = f'*f/2       ;                              end     ;   
    df = f-fc ; slope0 = gc*p    ;   dff = alpha*slope0 ;  pLen=norm(p) ;
    if (rcode~=3) ;                fOld  = 0                 ;   end    ;                     
    if (rcode==3)&((f>=fOld)|(df>dff))                                  ;
        not_done=false  ; x=x_old    ; f=fOld         ; del=del/2       ;
    elseif df >= dff    ; ndx = max( abs(p)./max(abs(x),1) )            ;
        if ndx < StpTol ; NoCh=true  ; not_done=false ; x=xc ; f=fc     ;
        else            ; rcode=2                                       ; 
            del_m = -(slope0/2)/(df-slope0)*pLen                        ;
            del = min( max(0.1*del,del_m),del/2 )                       ;
        end                                                             ;
    else                                                                ;
        dfP   = slope0 + p'*Lc*Lc'*p/2                                  ;
        cond1 = (abs(dfP-df)<=(abs(df)/10)) | (df<=slope0)              ;
        cond2 = ~NUsed              ;  cond3 = del<=0.99*MXStp          ;
        if (rcode~=2) & cond1 & cond2 & cond3                           ;
            rcode=3     ; x_old=x   ;  fOld=f   ; del=min(2*del,MXStp)  ;
        else            ; NoCh=false            ; not_done=false        ;
            if pLen>0.99*MXStp      ;  MXed=true             ;  end     ;
            if df >= dfP/10         ;  del=del/2                        ; 
            elseif df <= 0.75*dfP   ;  del = min( del*2, MXStp )        ;
            end                                                         ;
        end                                                             ;
    end                                                                 ;
end                                                                     ;