function [v,ord] = eval_pade_gen_coef(jind,lambdaind,P)
% 
%  function [v,ord] = eval_pade_gen_coef(jind,lambdaind,P)
%  =======================================================
%
%  obtains the coefficients of a Pade form of 
%  a finite difference approximation where 
%  jind      = indices of neighbors
%  lambdaind = derivative order of corresponding neighbors
%  P         = the order of derivative being approximated
%  v         = the desired coefficients
%  ord       = order of approximation

%  (c) 2011 by Tomas Co
%   @ Michigan Technological University 
%

    Q   = length(jind)-1        ;    Q1  = Q+1                      ;
    ord = Q1-P                  ;    cnt    = -1                    ;
    n0  = [0:Q]'                ;    kk  = ones(Q1,1)               ;
    ay  = kk*jind               ;    ax  = n0*kk'-kk*lambdaind      ; 
    G   = []                    ;
    for i = 1:Q1
        Grow = [] ;
        for j = 1:Q1
            Grow = [Grow, eval_gamma(ax(i,j),ay(i,j))]              ;
        end
        G = [G;Grow]                                                ;
    end
    ee  = zeros(Q1,1)           ;    ee(P+1)= 1                     ;
    v   = G\ee                  ;    
    lead_err = 0;
    while abs(lead_err)<=1e-10
        cnt     = cnt + 1       ;    w = []                         ;
        for i = 1:Q1
            w   = [w, eval_gamma(Q1+cnt-lambdaind(i),jind(i))]      ;
        end
        lead_err = w*v          ;
    end
    ord = ord + cnt;
    %
    % =============================
    function gamma = eval_gamma(r,s)
    if      r<0     ,   gamma = 0                                   ;
    elseif  r==0    ,   gamma = 1                                   ;
    else    gamma = s^r/factorial(r)                                ;
    end