function [x,y,tri] = exer_tank1_mesh(a,b,c,d,dL)
%
%  function [x,y,tri] = exer_tank1_mesh(a,b,c,d,dL)
%  =================================================
%   
%   triangulates a quarter cross-section 
%   of a tank with rectangular hole
%   in the upper left section
%
%        b +-----+ a
%          |     |
%          |     |
%          |     |
%    c +---+     |
%      |         |
%      |         /
%      |       /
%    d +----/
%
%       dL = characteristic length of elements
% Example: 
%  [x,y,tri] = exer_tank1_mesh([1;2.5],[0.25;2.5],[0;1.5],[0,0],0.05);

%   (c) 2011 by Tomas Co
%   Michigan Technological University

xx = 1; yy = 2;
R = a(xx)-d(xx);
[x,y] = circle_arc_mesh(R,-pi/2,0,dL);
y = y + R + d(yy);
x = x + d(xx);
cd = [d(xx);d(yy)+R+dL];
[rx,ry] = simple_rect_mesh(cd(xx),cd(yy),a(xx),c(yy),dL);
x = [x,rx];
y = [y,ry];
keep = y<c(yy);
x = x(keep); 
y = y(keep);
nn = ceil((b(xx)-c(xx))/(0.99*dL));
if nn<2
    x = [x, c(xx)];
    y = [y, c(yy)];
else
    Lix = linspace(c(xx),b(xx),nn);
    Lix(end) = [];
    Liy = c(yy)*ones(size(Lix));
    x = [x,Lix];
    y = [y,Liy];
end
[rx,ry] = simple_rect_mesh(b(xx),c(yy),a(xx),a(yy),dL);
x = [x,rx];
y = [y,ry];
tri = delaunay(x,y);
remove = x<=(b(xx)+dL*1e-2) & y>=(c(yy)-dL*1e-2);
remove = sum(remove(tri),2)==3;
tri = tri(~remove,:);

