function [x,y,u,Ind] = fem_cylinder()
%   function [x,y,u,Ind] = fem_cylinder()
%   ======================================
%   FEM test for cylinder using axisymmetric solution
%   

%   (c) 2011 Tomas Co
%   Michigan Technological University
%
    R = 1;
    L = 1;
    
%   Obtain Mesh Data
    %   ----------------
    %   DBC = indicator for Dirichlet conditions    ( 0=no, 1=yes )
    %   NBC = indicator for Neumann conditions      ( 0=no, 1=yes )
    %
    [x,y,Ind]   = rectangle_mesh(0,0,R,L,100,100);   % triangulation
    Nnodes  = length(x)                         ; % number of nodes
    P           = [x;y]                         ;   % node positions
    DBC         = sparse(Nnodes,1)              ; 
    NBC         = sparse(Nnodes,1)              ;
    RBC         = sparse(Nnodes,1)              ;
    DBC1        = x==R                          ;   % R
    DBC2        = y==0 | y==L                   ;   % z=0 and z=L
    DBC         = DBC1 | DBC2                   ;
    NBC         = (x==0)                        ;   % center, exit
    %
    %   Set Boundary Conditions
    %   -----------------------
    DBCval  = sparse(Nnodes,1)                  ;
    NBCval  = sparse(Nnodes,1)                  ;
    RBCval  = sparse(Nnodes,1)                  ;
    RBCmuval= sparse(Nnodes,1)                  ;
    DBCval(DBC1) = 100                          ; % get Dirich BCs
    DBCval(DBC2) = 30                           ; % get Dirich BCs
    %
    %   Implement FEM
    %   -------------
    %   linfem = linear FEM function
    %   @pdeM, @pdeb, @pdeg, @pdeh = function handles
    %           for M, b, g and h, respectively
    %
    u       = linear_2d_fem( P, Ind, DBC, DBCval, ...
                NBC, NBCval, ...
                RBC, RBCval, RBCmuval, ...
                @pdeM2, @pdeb2, ...
                @pdeg2, @pdeh2)                  ;
    function M = pdeM2(x,y)
            M = x*eye(2)  ;
    function b=pdeb2(x,y)
            b = [1;0]     ;
    function g=pdeg2(x,y)
            g = 0         ;
    function h=pdeh2(x,y)
            h = 0         ;