function fem_exer_streamline(vx,vy,xgrad,ygrad,choles,rholes,nlines)
%===================================================================
%  function fem_exer_streamline(vx,vy,xgrad,ygrad,choles,rholes,nlines)
%  
%  Script file to draw streamlines for the potential flow across an
%  array of horizontal cyclinder
%
%  It sssumes top and bottom plate have length L=2 units 
%  separated Y=1 units (modify program as needed)
%====================================================================

% (c) 2011 by Tomas Co
% @ Michigan Technological University
%
%   Build a new mesh and then interpolate values
%
    xmin    = 0 ;   xmax    = 2     ;   num_xvals = 400 ;
    ymin    = 0 ;   ymax    = 1/2   ;   num_yvals = 200 ;
    xrange  = linspace(xmin,xmax,num_xvals)             ;
    yrange  = linspace(ymin,ymax,num_yvals)             ;
    [xr,yr] = meshgrid(xrange,yrange)                   ;  
    urx     = griddata(xgrad,ygrad,vx,xr,yr)            ;  
    ury     = griddata(xgrad,ygrad,vy,xr,yr)            ;  
%                                              
%   Replace values inside the hole with NaN
%
    ncirc   = length(rholes)                            ;
    th      = linspace(0,2*pi,100)                      ;
    axis([xmin xmax ymin ymax])                         ;
    for i=1:ncirc
        cx              = choles(1,i)                   ; 
        cy              = choles(2,i)                   ;
        r               = rholes(i)                     ;
        rxy             = sqrt((xr-cx).^2+(yr-cy).^2)   ;
        r_in_hole       = rxy<=r                        ;
        urx(r_in_hole)  = NaN                           ;
        ury(r_in_hole)  = NaN                           ;
        xcirc           = r*cos(th)+cx                  ;
        ycirc           = r*sin(th)+cy                  ;
        hold on
        plot(xcirc,ycirc,'-r','linewidth',2)            ;
        plot([0,2],[0,0],'-r','linewidth',2)            ;
        plot([0,2],[1,1],'-r','linewidth',2)            ;
        hold off
    end
%  
%   Draw the streamlines
%    
    xstart  = (xmin+0.05)*ones(nlines,1)                ;
    ystart  = linspace(0,1,nlines)                      ;
    streamline(xr,yr,urx,ury,xstart,ystart)             ;
    axis equal;    shg                                  ;