function [x,y,u,uact,Ind] = fem_sq_test2()
%   function [x,y,u,uact,Ind] = fem_sq_test2()
%   ======================================
%   FEM test for square domain using SUPG method
%   

%   (c) 2011 Tomas Co
%   Michigan Technological University
%
    %   Obtain Mesh Data
    %   ----------------
    %   DBC = indicator for Dirichlet conditions    ( 0=no, 1=yes )
    %   NBC = indicator for Neumann conditions      ( 0=no, 1=yes )
    %
    [x,y,Ind]   = squared_mesh_example(10)      ;   % triangulation
    Nnodes  = length(x)                         ; % number of nodes
    P           = [x;y]                         ;   % node positions
    DBC         = sparse(Nnodes,1)              ; 
    NBC         = sparse(Nnodes,1)              ;
    RBC         = sparse(Nnodes,1)              ;
    DBC         =   (x==-1) | (x==1) |...
                    (y==-1) | (y==1)            ;   % north, south,
    %
    %   Set Boundary Conditions
    %   -----------------------
    DBCval  = sparse(Nnodes,1)                  ;
    NBCval  = sparse(Nnodes,1)                  ;
    RBCval  = sparse(Nnodes,1)                  ;
    RBCmuval= sparse(Nnodes,1)                  ;
    xdbc    = x(DBC==1)                         ; % x @ Dirich nodes
    ydbc    = y(DBC==1)                         ; % y @ Dirich nodes
    DBCval(DBC==1) = 1.5*xdbc.*ydbc + ...
                5*exp(-4*(xdbc.^2+ydbc.^2))     ; % get Dirich BCs
    %
    %   Implement FEM
    %   -------------
    %   linfem = linear FEM function
    %   @pdeM, @pdeb, @pdeg, @pdeh = function handles
    %           for M, b, g and h, respectively
    %
    u       = linear_2d_supg( P, Ind, DBC, DBCval, ...
                NBC, NBCval, ...
                RBC, RBCval, RBCmuval, ...
                @pdeM2, @pdeb2, ...
                @pdeg2, @pdeh2)                  ;
    %        
    %   Plot and compare with actual values
    %   -----------------------------------
    uact    = (1.5*x.*y + 5*exp(-4*(x.^2+y.^2)))' ; % exact u
    errors  = u - uact                          ; 
    figure(1); plot3(x,y,u,'o',x,y,uact,'*')    ; view([-45,60]);
    figure(2); plot3(x,y,errors,'o')            ; view([-45,60]);
    
    function M = pdeM2(x,y)
            a = 1000        ;
            M = 1/a*eye(2)  ;
    function b=pdeb2(x,y)
            b = [-2;3]      ;
    function g=pdeg2(x,y)
            g = 0           ;
    function h=pdeh2(x,y)
        a = 1000                                        ;
        alpha = (320/a)*(x^2+y^2)-80*(3*y/2-x+1/a)      ;
        h = -(3/2)*(3*x-2*y)-alpha*exp(-4*(x^2+y^2))    ;