function [x,y,u,uact]=fem_sqc_test1()
%  function [x,y,u,uact]=fem_sqc_test1()
%  =====================================
%   FEM test for square with circular hole used in Example 14.3
%

%   (c) 2005,2011 Tomas Co
%   Michigan Technological University
%
    %   Obtain Mesh Data
    %   ----------------
    %   DBC = indicator for Dirichlet conditions    ( 0=no, 1=yes )
    %   NBC = indicator for Neumann conditions      ( 0=no, 1=yes )
    %
    [x,y,Ind]   = squared_annulus_mesh_example(0.2,0.5,10); 
                                                    % triangulation
    x           = x+0.5                         ;
    y           = y+0.5                         ;
    P           = [x;y]                         ;   % node positions
    r           = sqrt((x-0.5).^2 + (y-0.5).^2) ;   % radius
    Nnodes      = length(x)                   ; % number of nodes
    DBC         = sparse(Nnodes,1)           ; 
    NBC         = sparse(Nnodes,1)           ;
    RBC         = sparse(Nnodes,1)           ;
    DBC         =   (x==0) |...
                    (y==0) | (y==1) | ...
                    (abs(r-0.2)<1e-6)           ;   % north, south,
                                                    % west + circle  
    NBC         = (x==1)                        ;   % east
    %
    %   Set Boundary Conditions
    %   -----------------------
    DBCval  = sparse(Nnodes,1)                  ;
    NBCval  = sparse(Nnodes,1)                  ;
    RBCval  = sparse(Nnodes,1)                  ;
    RBCmuval  = sparse(Nnodes,1)                  ;
    xdbc    = x(DBC==1)                         ; % x @ Dirich nodes
    ydbc    = y(DBC==1)                         ; % y @ Dirich nodes
    DBCval(DBC==1) = 5.3*xdbc.^2.*ydbc + ...
                2*sin(2*pi*xdbc)                ; % get Dirich BCs
    xnbc    = x(NBC==1)                         ; % x @ Neumann nodes
    ynbc    = y(NBC==1)                         ; % y @ Neumann nodes
    NBCval(NBC==1) = (6+xnbc.^2.*ynbc.^2)*...
                2*pi.*cos(2*pi*xnbc) +...
                31.8*xnbc.*ynbc + ...
                5.3*xnbc.^3.*ynbc.^3+...
                1.325*xnbc.^2                   ; % get Neumann BCs
    %
    %   Implement FEM
    %   -------------
    %   linfem = linear FEM function
    %   @pdeM, @pdeb, @pdeg, @pdeh = function handles
    %           for M, b, g and h, respectively
    %
        u  = linear_2d_fem( P, Ind, DBC, DBCval, ...
                NBC, NBCval, ...
                RBC, RBCval, RBCmuval,...
                @pdeM, @pdeb, ...
                @pdeg, @pdeh)                               ;
    %        
    %   Plot and compare with actual values
    %   -----------------------------------
    uact    = ((5.3*x.^2.*y) + 2*sin(2*pi*x))'  ; % exact u
    errors  = u - uact                          ; 
    figure(1); plot3(x,y,u,'o',x,y,uact,'*')    ; view([-45,60]);
    figure(2); plot3(x,y,errors,'o')            ; view([-45,60]);
    
    function    M = pdeM(x,y)        
        M=[(3+(x*y)^2/2),1/4;1/4,2]                         ;
    function    b = pdeb(x,y)
        b = [-x+1;-y+1]                                     ;
    function    g = pdeg(x,y)
        g = (x-y)^2                                         ;
    function    h = pdeh(x,y)
       theta   = 2*pi*x                                     ;
        s       = sin(theta)                                ;
        c       = cos(theta)                                ;
        alpha   = 4*pi*(1+x*y^2-x)                          ;
        beta    = -24*pi^2 - (2*pi*x*y)^2 + 2*(x-y)^2       ;
        gamma   = 21.2*x^2*y^3 - 10.6*y^2*x^3 + ...
                y*(5.3*x^4 - 15.9*x^2 + 10.6*x + 31.8) + ...
                5.3*x*(1 + x)                               ;
        h       = -(alpha*c +  beta*s + gamma)              ;