function  x = gmres_method(A,b,x0,tol)
%
%  function  x = gmres_method(A,b,x0,tol)
%  ======================================
%
%  Using the GMRES to find the solution of Ax=b
%  where x0 is the initial guess and tol is tolerance (default=1e-8)

%  (c)2009 by Tom Co
%  @ Michigan Technological University
%
    n = size(A,2)                                                   ;
    if nargin<4         , tol = 1e-8            ; end               ;
    if nargin<3         , x0 = zeros(n,1)       ; end               ;
    r0 = b-A*x0         ; nr = norm(r0)         ; u = r0/nr         ; 
    U = [u]             ; Q = [1]               ; R = []            ;
    for i=1:n                                                       ;
        w = A*u         ; h = U'*w              ; p = w-U*h         ; 
        alpha = norm(p) ; r = Q*h               ; R = [R,r]         ; 
        sw = 1                                                      ;            
        if (i<n||alpha>tol)                                         ;
            u = p/alpha             ; U = [U,u]                     ;
            R(i+1,i) = 0                                            ; 
            c = r(end)              ; s = alpha                     ; 
            rho = norm([c s])       ; R(end-1,end) = rho            ;
            z = [c s;-s c]/rho      ; Q(end+1,end+1) = 1            ;
            Q([end-1,end],:) = z*Q([end-1,end],:)                   ;
            if abs(Q(end,1)*nr)<tol ; break     ; end               ;
        else                                                        ;
            sw = 0                  ; break                         ;
        end                                                         ;
    end                                                             ;
    if sw == 1                                                      ; 
        R(end,:) = []   ; Q(end,:) = [] ; U(:,end) = []             ;
    end                                                             ;
    z = nr*Q(:,1)       ; y = z(i)/R(i,i)                           ;
    for j=i-1:-1:1                                                  ;
        f = (z(j)-R(j,j+1:end)*y)/R(j,j)    ;   y = [f;y]           ;
    end                                                             ;
    x = x0 + U*y                                                    ;