function [T,J]=jordan_decomp(A,tol)                        
%                                                           ;
%	function [T,J]=jordan_decomp(A)                         ;
%	function [T,J]=jordan_decomp(A,tol)                     ;
%	====================================                    ;
%                                                           ;
%	determines modal matrix such that T^(-1)AT=J            ;
%	is in Jordan canonical form                             ;
%                                                           ;
% tol is tolerance (default: 1e-6)                          ;
% Note: uses Matlab functions eig() and svd()               ;

%	(c) 2009 Tomas Co                                       ;
%                                                           ;
    if nargin < 2   ; tol = 1e-6        ; end               ;    
	n = size(A,1)   ; [vv,dd] = eig(A)  ; dd = [diag(dd)]'  ;
    dd = tol*round(dd/tol)                                  ;
    d = []          ; m = []            ; T = []            ;                            
    while ~isempty(dd)                                      ;
        ev = dd(1)              ; v = vv(:,1)               ;
        b = (abs(dd-ev)<tol)    ; sb = sum(b)               ;       
        if sb==1                                            ;  
            T = [T,v]                                       ;  
        else                                                ;
            d = [d,ev]          ; m = [m,sb]                ;
        end                                                 ;
        dd(b) = []              ; vv(:,b) = []              ;
    end                                                     ;
    for i = 1:length(d)                                     ;
        dA = A-d(i)*eye(n)      ; AA = dA                   ;
        nmi = n-m(i)            ; rA = n    ; Nik = []      ; 
        for pp = 1:n                                        ;
            rAA = rank(AA,tol)  ; drank = rAA - nmi         ; 
            AA = AA*dA          ; Nik = [Nik, rA-rAA]       ;
            if drank == 0       ; break     ; end           ;
            rA = rAA                                        ;
        end                                                 ;
        ord = Nik(pp)                                       ;
        for k = pp-1:-1:1                                   ;
            ord = [max( 0, Nik(k)-sum(ord) ), ord]          ;
        end                                                 ;
        for k = 1:pp                                        ;
            gamma = ord(k)                                  ;
            if gamma > 0                                    ;
                for j = 1:gamma                             ;
                    T = [T,chain(A,d(i),k,tol)]             ;
                end                                         ;
            end                                             ;
        end                                                 ;
    end                                                     ;
    J = inv(T)*A*T                                          ;

    function	VC = chain(A,lambda,r,tol)                  ;
%   =======================================                 ;
%                                                           ;
	n   = size(A,1)             ; dA = A-lambda*eye(n)      ;
	G1  = dA^(r-1)              ; G2 = G1*dA                ;
	G   = [G1,-eye(n);G2,zeros(n)]                          ;
    [U,S,V] = svd(G)            ; rr=sum(diag(S)>tol)       ;
    vv  = V(:,[rr+1:2*n])*randn(2*n-rr,1)                   ;
	v   = vv(1:n)               ; v = v/norm(v)             ;
    VC  = [v]                                               ;
    for i = 1:r-1                                           ;
        v = dA*v                ; VC = [v,VC]               ;
    end                                                     ;