function    [x,f,NoCh,MXed,lamb] = lines( Fx, xc, fc, gc, pn, ...
                                     MXstep, StpTol, nsolve, pars )
%
%  function [x,f,NoCh,MXed,lamb]=lines(Fx,xc,fc,gc,pn,...
%                                       MXstep,StpTol,pars)
%  ============================================================
%  line search for Newton minimization stepsize
% (adapted from Dennis and Schnabel, "Numerical Methods for Uncon-
%  strained Optimization and Nonlinear Equations",Prentice Hall,1983)

%   (c) 2007 by Tom Co
%   @ Michigan Technological University

    alpha = 1e-1    ;   MXed = false    ;   NLen = norm(pn)             ;
    pn = pn*min(1,MXstep/NLen)          ;   NLen = min(MXstep,NLen)     ;
    slope0 = gc*pn ;   lambMN = StpTol/max(abs(pn)./max(abs(xc),1))     ;
    NoCh = false    ;   lamb = 1        ;   not_done = true             ;
    if nargin <8    ;   nsolve = false  ;   end                         ;     
    while not_done                                                      ;
        x  = xc + lamb*pn                                               ;   
        if nargin>8 ; f=feval(Fx,x,pars);   else; f=feval(Fx,x);  end   ;                                                             ;
        if nsolve   ; f=f'*f/2                                 ;  end   ;
        if  f <= (fc + alpha*lamb*slope0)                               ;
            if (lamb==1)&(NLen>0.99*MXstep)     ;  MXed=true   ;  end   ;
            not_done  = false                                           ;
        elseif ( lamb < lambMN )                                        ;
            x = xc  ;   f = fc  ;   NoCh = true ;   not_done = false    ;
            disp( 'line search step too small ! ' )                     ;
        else                                                            ;
            if lamb==1  ;   lamb_m = -(slope0/2)/(f-fc-slope0)          ;
            else                                                        ;
                A = [ [1;-lamb_p]/lamb^2 , [-1;lamb]/lamb_p^2   ]       ;
                B = [ f-fc-slope0*lamb  ; fp-fc-slope0*lamb_p ]         ;
                v = A*B/( lamb-lamb_p )  ; a = 3*v(1)  ;  b = v(2)      ;
                if  a==0    ;  lamb_m = -slope0/( 2*b )                 ;
                else        ;  c = sqrt(b^2-a*slope0)  ; lamb_m=(c-b)/a ;
                end                                                     ;
                lamb_m = min( lamb/2, lamb_m )                          ;
            end                                                         ;
            lamb_p = lamb   ;  fp = f    ; lamb = max(0.1*lamb,lamb_m ) ;
        end                                                             ;
    end                                                                 ;