function  [H1,H2,Q,lambda,m] = linode_mats(A,x0,tol)
%
%   function  [H1,H2,Q,lambda,m] = linode_mats(A,x0,tol)
%   ====================================================   
%
%       for solution of dx/dt = Ax + b,
%       
%           x = H1*g + H2*int( kron(g,b) )
%       (note:  eigenvalues are sorted for 
%               real parts in ascending order,
%               example: [-2,-1,0,1+2i,1-2i,2] )           
%       lambda = list of eigenvalues
%       m      = multiplicities of the eigenvalues
%       tol    = tolerance

%   (c) 2006 by Tomas Co  @ Michigan Tech University
%

    if nargin<3, tol=1e-6; end;
    h  = [eig(A)]'  ;   n = size(A,1) ;  lambda  = [] ; 
    m  = []         ;   Q = []        ;  h = sort(h)  ;
    while length(h) > 0                               ;
        h1      = round(h(1)/(tol*10))*(tol*10)       ;
%        if abs(real(h1))<tol
%           if abs(imag(h1))<tol
%               h1 = 0                                 ;
%           else
%               h1 = imag(h1)                          ;
%           end
%        else
%           if abs(imag(h1))<tol
%               h1 = real(h1)                          ;
%           end
%        end
        lambda  = [ lambda  ; h1 ]                    ;
        hh      = abs(h-h1)<tol                       ;
        m       = [ m ; length(h(hh))]                ;
        h       = h( ~hh )                            ;
    end                                               ;
    [lambda,m]
    for i = 1:length(lambda)                          ;
        Qi  = zeros(m(i),n)                           ;
        for j = 0:m(i)-1                              ;
            for ell=j:n-1                             ;
                Qi(j+1,ell+1) = lambda(i)^(ell-j)...
                    *factorial(ell)/factorial(ell-j)  ;
            end                                       ;
        end                                           ;
        Q   = [Q;Qi]                                  ;
    end                                               ;
    H1  = x0         ; H2  = eye(n)                   ;
    for i=1:n-1                                       ;
        H1  = [x0,A*H1]                               ;
        H2  = [H2,A^i]                                ;
    end                                               ;
    H1  = H1*inv(Q)                                   ;
    H2  = H2*kron(inv(Q),eye(n))                      ;
    H1  = round(H1/(tol*10))*tol*10                     ;
    H2  = round(H2/(tol*10))*tol*10                     ;
    Q   = round(Q/(tol*10))*tol*10                      ;