function tau = new_stab_tau(p,b,mu,sigma)
%
%   function tau = new_stab_tau(p,b,mu)
%   ===================================
%
%       tau = SUPG stabilization parameter
%       p = triangle vertices
%       b = velocity vector
%       mu = diffusivity

%       (c) 2011 by Tomas Co
%       @ Michigan Technological University
%
    db      = norm(b)    ;    nb      = b/norm(b)       ;
    for i=1:3
        switch i
            case 1
                s = triang_h(p(:,1),p(:,2),p(:,3),nb)   ;
            case 2
                s = triang_h(p(:,2),p(:,3),p(:,1),nb)   ;
            case 3
                s = triang_h(p(:,3),p(:,1),p(:,2),nb)   ;
        end
        lambda = s(2)                                   ;
        if lambda<=1 && lambda>0        
            ell       = abs(s(1))                       ;
            break
        end
    end
    %
    %   based on Shakib method
    %
    t1 = (2*db/ell)^2 ;
    t2 = 9*(4*mu/(ell^2))^2 ;
    t3 = sigma^2 ;
    tau      = 1/sqrt( t1 + t2 + t3)                    ;

    function s=triang_h(pa,pb,pc,nb)
        vcb     = pc-pb                                 ;
        vca     = pc-pa                                 ;
        if abs(det([nb vcb]))<1e-6
            s = [1.2;0];
        else
            s       = [nb vcb]\vca                      ;
        end