function [x,y,tri] = rect_circ_mesh(ne_pt,sw_pt,choles,rholes,dL)
%
% function [x,y,tri] = rect_circ_mesh(ne_pt,sw_pt,c,r,dL)
% ====================================================
%
%   triangulation for a rectangle with circular hole
%
%       ne_pt = northeast point of rectangle
%       sw_pt = southwest point of rectangle
%       choles     = center of circular holes
%       rholes     = radius of circular holes
%       dL    = characteristic length of finite element

%   (c) 2011 by Tomas Co
%   @ Michigan Technological University
%

    x1 = ne_pt(1);
    y1 = ne_pt(2);
    x2 = sw_pt(1);
    y2 = sw_pt(2);    
    nrows = ceil((x2-x1)/(0.99*dL));
    ncols = ceil((y2-y1)/(0.99*dL));
    xrange = linspace(x1,x2,nrows);
    yrange = linspace(y1,y2,ncols);
    [xa,ya] = meshgrid(xrange,yrange);
    x = xa(:)';
    y = ya(:)';
%
%   Remove holes
%
    nholes = size(choles,2);
    for i=1:nholes
        r2 = rholes(i);
        c2x = choles(1,i);
        c2y = choles(2,i);
        d_from_hole_center = sqrt( (x-c2x).^2 + (y-c2y).^2 ); 
        inside_hole =   d_from_hole_center < (r2 + dL/6) ;
        x = x(~inside_hole);
        y = y(~inside_hole);
        %
        nth = ceil( 2*pi*r2/(0.99*dL) );
        th = linspace(0,2*pi,nth);
        th(end)=[];
        xhole = (r2)*cos(th)+c2x;
        yhole = (r2)*sin(th)+c2y;
        x =[x,xhole];
        y =[y,yhole];
    end
    tri = delaunay(x,y);
    keep = ones(size(tri,1),1);
    for i=1:nholes
        r2 = rholes(i);
        c2x = choles(1,i);
        c2y = choles(2,i);
        d_from_hole_center = sqrt( (x-c2x).^2 + (y-c2y).^2 ); 
        at_hole_boundary =   d_from_hole_center < (r2 + 1e-8) ;
        keep = keep & sum(at_hole_boundary(tri),2)<3;
    end
    tri = tri(keep,:);