function    [t,y] = rk4(t0,tfinal,y0,dt)
%   
%   function    [t,y] = rk4(t0,tfinal,y0,dt)
%   ========================================
%
%   Fixed step 4th Order Runge Kutta method 
%   
%       t0      = initial time
%       tfinal  = final time
%       y0      = initial value
%       dt      = time step
%
%       (user can modify the function
%        near the end of the file)

%   (c) 2006 by Tomas Co
%   @ Michigan Technological University

    t   = [t0]                                  ;
    y   = [y0']                                 ;
    tk  = t0                                    ;
    yk  = y0                                    ;
    while tk < tfinal
        
        h       = min(dt,tfinal-tk)             ;
        delta1  = h*f(tk      , yk            ) ;
        delta2  = h*f(tk + h/2, yk + delta1/2 ) ;
        delta3  = h*f(tk + h/2, yk + delta2/2 ) ;
        delta4  = h*f(tk + h  , yk + delta3   ) ;
    
        yk      = yk + ( delta1 + 2*delta2 ...
                         + 2*delta3 + delta4)/6 ;
        tk      = tk + h                        ;
        t       = [t;tk]                        ;
        y       = [y;yk']                       ;
    end
    
    function dy = f(t,y)
        y1      = y(1);
        y2      = y(2);
        
        dy1  = -20*y1 -5*y2  ;
        dy2  = 10*y1 - 35*y2 ;
        dy   = [dy1;dy2]     ;
