function [x,y,tri] = squared_annulus_mesh_example(inner_rad,...
                        outer_rad,levels)
%
%   function [x,y,tri]=squared_annulus_mesh_example(...
%                      inner_rad,outer_rad,levels,psize)
%   =====================================================  
%   
%       radius = maximum radius
%       levels = number of concentric circles

%   (c) 2005, 2011 by Tomas Co
%   Michigan Technological University

%
% Initialize parameters
% =====================
%  psize is the number of equal pie pieces of circle
%  dr1 is the planned approximate distance among nodes
%
    psize   = 6                                     ; 
    dr1     = (outer_rad-inner_rad)/levels          ;
    in_level= ceil(inner_rad/dr1)                   ;
    theta0  = 2*pi/psize                            ;
    theta   = [0:in_level*psize-1]*theta0/in_level  ;
    
% Gathering boundary points
% =========================
%
%   a) inner circular points
%
    x_cir       = [inner_rad*cos(theta)]            ;
    y_cir       = [inner_rad*sin(theta)]            ;
%  
%   b) outer square points
%
    diameter= 2*outer_rad                           ;
    nside   = ceil(diameter/dr1)                    ;
    dr      = diameter/nside                        ;
    aa      = (0:nside)*dr-outer_rad                ;
    x_box   = aa                                    ;
    y_box   = ones(1,length(aa))*outer_rad          ;
    x_box   = [x_box,aa]                            ;
    y_box   = [y_box,-ones(1,length(aa))*outer_rad] ;
    aa(1)   = []                                    ;
    aa(nside)=[]                                    ;
    y_box   = [y_box,aa]                            ;
    x_box   = [x_box,ones(1,length(aa))*outer_rad]  ;
    y_box   = [y_box,aa]                            ;
    x_box   = [x_box,-ones(1,length(aa))*outer_rad] ;
%
% Gathering internal points
% =========================
%
%   a) generating points in growing radius
%      (with arc distance close to dr1)
%
    metalevels = ceil(sqrt(2)*(in_level+levels))    ;
    nc      = 1:metalevels-2                        ;
    x       = []                                    ;
    y       = []                                    ;
    for ik = nc
        r       = inner_rad + dr1*ik                ;
        theta   = (0:(ik+in_level)*psize-1)*...
                        theta0/(ik+in_level)        ;
        x       = [x,r*cos(theta)]                  ;
        y       = [y,r*sin(theta)]                  ;
    end
%    
%   b) cropping points using boundary set by box
%
    tol     =   0.4                                 ;    
    aa      = (x>(outer_rad-dr1*tol))               ;
    x(aa==1)= []; y(aa==1)= []                      ;
    aa      = (x<-(outer_rad-dr1*tol))              ;
    x(aa==1)= []; y(aa==1)= []                      ;
    aa      = (y>(outer_rad-dr1*tol))               ;
    x(aa==1)= []; y(aa==1)= []                      ;
    aa=(y<-(outer_rad-dr1*tol))                     ;
    x(aa==1)= []; y(aa==1)= []                      ;
%
% Combining boundary and internal points
% ======================================
%
    x = [x_cir,x_box,x]                             ;
    y = [y_cir,y_box,y]                             ;
%
% Generate Delaunay triangular mesh
% =================================
%
    tri     = delaunay(x,y)                         ;
    vtri    = 1:size(tri,1)                         ;
    n_in    = length(x_cir)                         ;
%
% Removing triangles inside circular hole
% =======================================
% 
    vtri    = vtri(sum([tri<=n_in]')==3)            ;
    tri(vtri,:)= []                                 ;